import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { createElement as _createElement } from "@emotion/react";
import * as React from "react";
import { animated, useSprings } from "@react-spring/web";
import { useDrag } from "@use-gesture/react";
import { PanelEnlargeCtx, useMMenuBarOcup } from "./MainMenuBar";
import { DdMenu, DdMenuItem } from "./DropdownMenu";
import { IconChevronsLeft } from "./Icons";
import { MenuItem } from "./Menu";
import { useMeasure } from "./Hooks/use-measure";
import { useTheme } from "./Theme/Providers";
// const containerStyle: React.CSSProperties = {
//     position: 'relative',
// }
// const pageStyle: MotionStyle = {
//     top: 0,
//     flex: "none",
//     overflowX: "hidden",
// }
// const transition: AnimationOptions<any> = {
//     type: "spring",
//     bounce: 0,
// }
// const Contaier = React.forwardRef<
//     HTMLDivElement,
//     { children: React.ReactNode }
//     >((props, ref) => (
//     <div ref={ref} style={containerStyle}>
//         {props.children}
//     </div>
// ))
/**公用框架组件  Left左半列表摘要 + 右半边Console详细操作的页面。手机窄屏变换为轮播。
 * 【注意】useContext(PanelEnlargeCtx);应该定义在TwoHalfFrame组件的层次顺序之下的，而不是TwoHalfFrame之前，否则PanelEnlargeCtx不正常。
 * TwoHalfFrame应该放在路由器每个路由顶部，外部加个PanelEnlargeCtx.Provider配套控制本实例TwoHalfFrame。
 * 本组件是框架，要配合 <TwoHalfRightPanel/>组件 以及<PanelEnlargeCtx.Provider 这两个东西来协同完成框架页面构建。
 * 需要在页面路由器头上加<PanelEnlargeCtx.Provider  value={{ enlarge, setEnlarge，activecas，setActivecas }}>
 * 【特别】<PanelEnlargeCtx.Provider的提供位置非常严格，可能无效:初始化取值，还是后来能修改赋值?。
 * 手机模式TwoHalfFrame右边页面实际和左边页面并排在X轴线，依据水平动画X变换移动来展示的。所以有些依据定位组件可能不适合右边页面会依附于左边页面的无效情况！
 * */
export const TwoHalfFrame = React.forwardRef(({ children, rightPanel, leftStyle, rightStyle, enlarge: initEnlarge = 2, ...other }, ref) => {
    const theme = useTheme();
    const { barHeight, bottomCss, carousel, isBigSrc, touchScr, mobileList } = useMMenuBarOcup(!rightPanel);
    const [enlarge, setEnlarge] = React.useState((!rightPanel && initEnlarge === 0) ? 2 : initEnlarge);
    const containerRef = React.useRef(null);
    const { bounds } = useMeasure(containerRef);
    const { width, } = bounds;
    // const x = useMotionValue(0);
    //正常轮播状态下默认优先看右边页面。
    const [index, setIndex] = React.useState(1);
    /*和布局相关的：手机书评模式情况下：transformX(0)实际是显示右半边页面的。transformX(containerRef.current?.clientWidth)实际是显示左半边页面的。
    * */
    // const calculateNewX = () =>
    //     (!carousel? 0 : index===1? 0 : -1) * (containerRef.current?.clientWidth || 0);
    //两个页面拼凑:
    const childrens = [children, rightPanel];
    // const handleNext = () => {
    //     setIndex(index===1 ? 0 : 1);
    // }
    // const handlePrev = () => {
    //     setIndex(index===0 ? 1 : 0);
    // }
    //外部也可以操纵，轮播台当前显示右边或左边。 【失败】注入Context实际有问题=导致ref.setActivecas()可能为undefined!
    // React.useImperativeHandle(ref, () => ({
    // }));
    const getActivecas = () => {
        return (index === 0 ? 1 : 0);
    };
    //无法直接用index对外替换Activecas？
    const setActivecas = (i) => {
        setIndex(i === 0 ? 1 : 0);
    };
    //useLayoutEffect
    // React.useEffect(() => {
    //     x.set(0);
    //     //若用x.set(0);在autofoucus导致自动切换显示右边页面情况，无法正常。只好改成默认set(-1*?)
    //     x.stop();       //这个必须加上！
    //     const controls = animate(x, calculateNewX(), transition);
    //     return controls.stop
    // }, [carousel, containerRef.current?.clientWidth,  x])
    //已经默认，不需要外边initActivecas注入
    React.useEffect(() => {
        //轮播台 右边页面；
        setIndex(1);
    }, [rightPanel]);
    //非手机轮播台模式的左右两半边页面宽度比例： 左边没有=0，1：左边右边1比2， 2： 1比1， 3： 2比1， 4：右边没有。
    const leftpwp = enlarge === 0 ? 0 : (enlarge === 1 ? 33.4 : (enlarge === 4 ? 100 : (enlarge === 3 ? 66.6 : 50)));
    //轮播形态的【页面组织】动画和隐藏。
    const [springs, setSprings] = useSprings(2, i => {
        return getAnimationValues(i, index);
    });
    React.useEffect(() => {
        setSprings(i => getAnimationValues(i, index));
    }, [index, setSprings]);
    //最简易轮播台：只有左右两个页面，也不考虑嵌入在父辈轮播底下的需求。
    const bind = useDrag((state) => {
        const { active, movement: [mx, my], direction: [xDir, yDir], cancel, swipe: [swpx,], last, canceled, tap, event } = state;
        console.log("框拉", state, "active", active, "last:", last, "tap", tap);
        // event.stopPropagation();
        if (!active && last && !canceled && (Math.abs(swpx) > 0 || (Math.abs(mx) >= width * 0.03 && Math.abs(mx) > Math.abs(my)))) {
            let finalDir = xDir !== 0 ? xDir : mx > 0 ? 1 : -1;
            if (xDir === 0 && yDir !== 0)
                finalDir = 0; //允许水平方向滑动
            if (finalDir !== 0) {
                setIndex(0 === index ? 1 : 0);
                cancel();
            }
        }
        // api.start(i => {
        //     if (i < index - 1 || i > index + 1)
        //         return { display: 'none' };
        //     const x = (i - index) * width + (active ? mx : 0);
        //     const scale = active ? 0.97 : 1;
        //     return { x, scale, display: 'block' };
        // });
        setSprings(i => getAnimationValues(i, index));
    }, {
        swipe: { velocity: 0.1 },
        filterTaps: true,
        pointer: { mouse: !touchScr },
        // pointer: {touch: true, mouse: !touchScr, },      //嵌套Pager拉拽避免拉动父辈层和search input的cancel在电脑模式点击能清空的要求正好冲突了。
        // triggerAllEvents: true,       //嵌套在另外一个Pager底下的Pager设置true。
        // enabled: carousel,
    });
    /*预定义style={{}}无法通过css={{}}样式参数来覆盖，只好全都放入css={{}}。
    * */
    return (_jsxs(PanelEnlargeCtx.Provider, { value: { enlarge, setEnlarge,
            setActivecas: setActivecas, getActivecas: getActivecas,
        }, children: [_jsx("div", { className: "Thframe", ref: containerRef, css: {
                    overflowX: carousel ? "hidden" : undefined,
                    position: carousel ? 'relative' : 'static',
                    // position: 'relative',
                    // overflow: 'hidden',
                    width: "100%",
                    height: "100vh",
                    //height: "100%",
                }, ...other, children: springs.map((props, i) => {
                    const panStyle = i === 0 ? leftStyle : rightStyle;
                    return (_createElement(animated.div, { className: `Frm${index} ${i === 0 ? "LeftP" : "RightP"}`, ...bind(), key: i, style: {
                            touchAction: 'none',
                            display: "flex",
                            flexDirection: 'column',
                            flex: "none",
                            overflowY: i === 0 ? 'auto' : 'auto',
                            //单位 overflowY: i===0? 'hidden' : 'auto',
                            overflowX: carousel ? undefined : "hidden",
                            position: "absolute",
                            // display: 'block',
                            // top: 0,
                            // left: `${i * width}`,
                            // x: `${0 * width}`,
                            //x: active || gesture === 'offset' ? x : 0,
                            top: 0,
                            left: i === 0 ? 0 : carousel ? "100%" : mobileList ? '100%' : `${leftpwp}%`,
                            right: 0,
                            bottom: 0,
                            opacity: carousel ? props.opacity : undefined,
                            transform: carousel ? props.left.to(x => clamp(x)).to(x => `translateX(${x}%)`) : undefined,
                            height: i === 0 ? `calc(100vh - ${barHeight})` : carousel ? `calc(100vh - ${barHeight})` : '100%',
                            width: i === 0 ? ((carousel || mobileList) ? "100%" : `${leftpwp}%`) : (carousel ? "100%" : mobileList ? '0%' : `${100 - leftpwp}%`),
                        } },
                        _jsx(animated.div, { className: 'Thf_pane', css: {
                                overflowY: i === 0 ? undefined : 'auto',
                                marginBottom: i === 0 ? `${barHeight}` : undefined,
                                width: "100%",
                                height: "100%",
                                ...panStyle
                            }, tabIndex: index === i ? 0 : -1, "aria-hidden": i !== index, children: childrens[i] }, i)));
                }) }), (!carousel && enlarge === 4) &&
                _jsx("div", { className: "Larger", css: {
                        // display: 'inline-flex',
                        right: 0,
                        bottom: 0,
                        position: 'fixed',
                        zIndex: theme.zIndices.fixed
                    }, children: _jsxs(DdMenu, { label: "\u653E\u5927", tight: true, icon: _jsx(MenuItem, { variant: "ghost", component: "div", children: _jsx(IconChevronsLeft, {}) }), children: [_jsx(DdMenuItem, { label: "\u5DE6\u6EE1\u5C4F", onClick: (e) => { setEnlarge(4); } }), _jsx(DdMenuItem, { label: "1\u6BD42", onClick: (e) => { setEnlarge(1); } }), _jsx(DdMenuItem, { label: "1\u6BD41", onClick: (e) => { setEnlarge(2); } }), _jsx(DdMenuItem, { label: "2\u6BD41", onClick: (e) => { setEnlarge(3); } }), _jsx(DdMenuItem, { label: "\u53F3\u6EE1\u5C4F", onClick: (e) => { setEnlarge(0); } })] }) })] }));
});
function getAnimationValues(i, currentIndex) {
    if (0 === currentIndex) {
        if (0 === i)
            return { left: 0, immediate: false, opacity: 1 };
        else
            return { left: 100, immediate: false, opacity: 0 };
    }
    else {
        if (0 === i)
            return { left: -100, immediate: false, opacity: 0 };
        else
            return { left: -100, immediate: false, opacity: 1 };
    }
}
function clamp(x) {
    if (x > 50) {
        return 100;
    }
    if (x < -50) {
        return -100;
    }
    return x;
}
//# sourceMappingURL=TwoHalfFrame.js.map